@@ -89,7 +89,8 @@ ospf6_neighbor_create (u_int32_t router_id, struct ospf6_interface *oi)
             buf, oi->interface->name);
   on->ospf6_if = oi;
   on->state = OSPF6_NEIGHBOR_DOWN;
-  gettimeofday (&on->last_changed, (struct timezone *) NULL);
+  on->state_change = 0;
+  quagga_gettime (QUAGGA_CLK_MONOTONIC, &on->last_changed);
   on->router_id = router_id;
 
   on->summary_list = ospf6_lsdb_create (on);
@@ -154,7 +155,8 @@ ospf6_neighbor_state_change (u_char next_state, struct ospf6_neighbor *on)
   if (prev_state == next_state)
     return;
 
-  gettimeofday (&on->last_changed, (struct timezone *) NULL);
+  on->state_change++;
+  quagga_gettime (QUAGGA_CLK_MONOTONIC, &on->last_changed);
 
   /* log */
   if (IS_OSPF6_DEBUG_NEIGHBOR (STATE))
@@ -180,10 +182,19 @@ ospf6_neighbor_state_change (u_char next_state, struct ospf6_neighbor *on)
       (next_state != OSPF6_NEIGHBOR_EXCHANGE &&
        next_state != OSPF6_NEIGHBOR_LOADING))
     ospf6_maxage_remove (on->ospf6_if->area->ospf6);
+
+#ifdef HAVE_SNMP
+  /* Terminal state or regression */ 
+  if ((next_state == OSPF6_NEIGHBOR_FULL)  ||
+      (next_state == OSPF6_NEIGHBOR_TWOWAY) ||
+      (next_state < prev_state))
+    ospf6TrapNbrStateChange (on);
+#endif
+
 }
 
 /* RFC2328 section 10.4 */
-int
+static int
 need_adjacency (struct ospf6_neighbor *on)
 {
   if (on->ospf6_if->state == OSPF6_INTERFACE_POINTTOPOINT ||
@@ -551,7 +562,7 @@ inactivity_timer (struct thread *thread)
 
 /* vty functions */
 /* show neighbor structure */
-void
+static void
 ospf6_neighbor_show (struct vty *vty, struct ospf6_neighbor *on)
 {
   char router_id[16];
@@ -568,7 +579,7 @@ ospf6_neighbor_show (struct vty *vty, struct ospf6_neighbor *on)
   }
 #endif /*HAVE_GETNAMEINFO*/
 
-  gettimeofday (&now, NULL);
+  quagga_gettime (QUAGGA_CLK_MONOTONIC, &now);
 
   /* Dead time */
   h = m = s = 0;
@@ -612,7 +623,7 @@ ospf6_neighbor_show (struct vty *vty, struct ospf6_neighbor *on)
            ospf6_interface_state_str[on->ospf6_if->state], VNL);
 }
 
-void
+static void
 ospf6_neighbor_show_drchoice (struct vty *vty, struct ospf6_neighbor *on)
 {
   char router_id[16];
@@ -630,7 +641,7 @@ ospf6_neighbor_show_drchoice (struct vty *vty, struct ospf6_neighbor *on)
   inet_ntop (AF_INET, &on->drouter, drouter, sizeof (drouter));
   inet_ntop (AF_INET, &on->bdrouter, bdrouter, sizeof (bdrouter));
 
-  gettimeofday (&now, NULL);
+  quagga_gettime (QUAGGA_CLK_MONOTONIC, &now);
   timersub (&now, &on->last_changed, &res);
   timerstring (&res, duration, sizeof (duration));
 
@@ -641,7 +652,7 @@ ospf6_neighbor_show_drchoice (struct vty *vty, struct ospf6_neighbor *on)
            VNL);
 }
 
-void
+static void
 ospf6_neighbor_show_detail (struct vty *vty, struct ospf6_neighbor *on)
 {
   char drouter[16], bdrouter[16];
@@ -654,7 +665,7 @@ ospf6_neighbor_show_detail (struct vty *vty, struct ospf6_neighbor *on)
   inet_ntop (AF_INET, &on->drouter, drouter, sizeof (drouter));
   inet_ntop (AF_INET, &on->bdrouter, bdrouter, sizeof (bdrouter));
 
-  gettimeofday (&now, NULL);
+  quagga_gettime (QUAGGA_CLK_MONOTONIC, &now);
   timersub (&now, &on->last_changed, &res);
   timerstring (&res, duration, sizeof (duration));
 
@@ -801,7 +812,7 @@ ALIAS (show_ipv6_ospf6_neighbor,
        "Neighbor list\n"
        "Display details\n"
        "Display DR choices\n"
-      );
+      )
 
 DEFUN (show_ipv6_ospf6_neighbor_one,
        show_ipv6_ospf6_neighbor_one_cmd,
@@ -839,7 +850,7 @@ DEFUN (show_ipv6_ospf6_neighbor_one,
 }
 
 void
-ospf6_neighbor_init ()
+ospf6_neighbor_init (void)
 {
   install_element (VIEW_NODE, &show_ipv6_ospf6_neighbor_cmd);
   install_element (VIEW_NODE, &show_ipv6_ospf6_neighbor_detail_cmd);
@@ -878,7 +889,7 @@ ALIAS (debug_ospf6_neighbor,
        "Debug OSPFv3 Neighbor\n"
        "Debug OSPFv3 Neighbor State Change\n"
        "Debug OSPFv3 Neighbor Event\n"
-      );
+      )
 
 DEFUN (no_debug_ospf6_neighbor,
        no_debug_ospf6_neighbor_cmd,
@@ -913,7 +924,7 @@ ALIAS (no_debug_ospf6_neighbor,
        "Debug OSPFv3 Neighbor\n"
        "Debug OSPFv3 Neighbor State Change\n"
        "Debug OSPFv3 Neighbor Event\n"
-      );
+      )
 
 int
 config_write_ospf6_debug_neighbor (struct vty *vty)
@@ -929,7 +940,7 @@ config_write_ospf6_debug_neighbor (struct vty *vty)
 }
 
 void
-install_element_ospf6_debug_neighbor ()
+install_element_ospf6_debug_neighbor (void)
 {
   install_element (ENABLE_NODE, &debug_ospf6_neighbor_cmd);
   install_element (ENABLE_NODE, &debug_ospf6_neighbor_detail_cmd);
